import { NetworkType } from '../..';
export declare enum Platform {
    DESKTOP = 0,
    IOS = 1,
    ANDROID = 2
}
export declare enum WalletType {
    IOS = "ios",
    ANDROID = "android",
    EXTENSION = "extension",
    DESKTOP = "desktop",
    WEB = "web"
}
export interface AppBase {
    name: string;
    shortName: string;
    color: string;
    logo: string;
}
export interface ExtensionApp extends AppBase {
    id: string;
    link: string;
}
export interface WebApp extends AppBase {
    links: {
        [NetworkType.MAINNET]: string;
        [NetworkType.DELPHINET]?: string;
        [NetworkType.EDONET]?: string;
        [NetworkType.CUSTOM]?: string;
    };
}
export interface DesktopApp extends AppBase {
    deepLink: string;
}
export interface App extends AppBase {
    universalLink: string;
    deepLink?: string;
}
export interface PairingAlertWallet {
    name: string;
    shortName?: string;
    color?: string;
    logo?: string;
    enabled: boolean;
    clickHandler(): void;
}
export interface PairingAlertButton {
    title: string;
    text: string;
    clickHandler(): void;
}
export interface PairingAlertList {
    title: string;
    type: WalletType;
    wallets: PairingAlertWallet[];
}
export interface PairingAlertInfo {
    walletLists: PairingAlertList[];
    buttons: PairingAlertButton[];
    qrData: string;
}
export declare type StatusUpdateHandler = (walletType: WalletType, app?: PairingAlertWallet) => void;
export declare class Pairing {
    static getPlatfrom(): Promise<Platform>;
    static getPairingInfo(pairingPayload: {
        p2pSyncCode: string;
        postmessageSyncCode: string;
        preferredNetwork: NetworkType;
    }, statusUpdateHandler: StatusUpdateHandler, platform?: Platform): Promise<PairingAlertInfo>;
    private static getDesktopPairingAlert;
    private static getIOSPairingAlert;
    private static getAndroidPairingAlert;
    private static getWebList;
}
