import * as sodium from 'libsodium-wrappers';
import { MatrixClientEvent, MatrixClientEventType, MatrixClientEventMessageContent } from '../../matrix-client/models/MatrixClientEvent';
import { Storage } from '../../storage/Storage';
import { P2PPairingRequest } from '../..';
import { ExtendedP2PPairingResponse, P2PPairingResponse } from '../../types/P2PPairingResponse';
import { CommunicationClient } from './CommunicationClient';
export declare class P2PCommunicationClient extends CommunicationClient {
    private readonly name;
    readonly replicationCount: number;
    private readonly storage;
    private readonly iconUrl?;
    private readonly appUrl?;
    private readonly clients;
    private readonly KNOWN_RELAY_SERVERS;
    private readonly activeListeners;
    constructor(name: string, keyPair: sodium.KeyPair, replicationCount: number, storage: Storage, matrixNodes: string[], iconUrl?: string | undefined, appUrl?: string | undefined);
    getPairingRequestInfo(): Promise<P2PPairingRequest>;
    getPairingResponseInfo(request: P2PPairingRequest): Promise<P2PPairingResponse>;
    getRelayServer(publicKeyHash?: string, nonce?: string): Promise<string>;
    start(): Promise<void>;
    listenForEncryptedMessage(senderPublicKey: string, messageCallback: (message: string) => void): Promise<void>;
    unsubscribeFromEncryptedMessage(senderPublicKey: string): Promise<void>;
    unsubscribeFromEncryptedMessages(): Promise<void>;
    sendMessage(message: string, peer: P2PPairingRequest | ExtendedP2PPairingResponse): Promise<void>;
    deleteRoomIdFromRooms(roomId: string): Promise<void>;
    listenForChannelOpening(messageCallback: (pairingResponse: ExtendedP2PPairingResponse) => void): Promise<void>;
    sendPairingResponse(pairingRequest: P2PPairingRequest): Promise<void>;
    isTextMessage(content: MatrixClientEventMessageContent<any>): content is MatrixClientEventMessageContent<string>;
    isChannelOpenMessage(content: MatrixClientEventMessageContent<string>): Promise<boolean>;
    isSender(event: MatrixClientEvent<MatrixClientEventType.MESSAGE>, senderPublicKey: string): Promise<boolean>;
    private getAbsoluteBigIntDifference;
    private getRelevantRoom;
    private getRelevantJoinedRoom;
}
